﻿
local SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)

SpeakinSpell:PrintLoading("SpeakinSpell.lua")

-------------------------------------------------------------------------------
-- GENERAL INITIALIZATION AND MAIN ENTRY POINTS
-------------------------------------------------------------------------------


function SpeakinSpell:ResetToDefaults(FrameName) --TODOFUTURE: reset only the current frame's defaults (optionally)
	table.wipe( self:GetActiveEventTable() )
	
	-- stop networking
	self:Network_Disable()
	
	-- update/reset runtime data state
	self:InitRuntimeData()
	
	-- reset the default templates to restore previously redundant content
	self:Templates_Release()
	
	-- copy defaults into current settings using class-specific defaults
	self:InitDefaultSavedData()
	
	-- if the options GUI is showing right now, we need to refresh it to make it show the new data
	self:RefreshAllFrames()
	
	-- resume networking
	self:Network_Init()
	
	self:Print(L["Default settings restored"])
end




function SpeakinSpell:OnInitialize()
	local funcname = "OnInitialize"
	self:DebugMsg(funcname, "entry")

	-- called by the WoWAce framework during ADDON_LOADED
	-- which means saved variables have already been loaded... I think...
	-- it works fine for me to do the work of OnLoad here
	-- but a few users have periodically complained about their data becoming erased
	-- so I worried that I should switch from VARIABLES_LOADED to ADDON_LOADED
	-- and it does indeed seem risky to call InitRuntimeData here if the framework is
	-- calling this in response to ADDON_LOADED but I thought variables hadn't been loaded yet
	-- 3.3.3.02 changed to running OnLoad as in-line code that executes as this file gets loaded
	-- which is listed last in the TOC
end


function SpeakinSpell:OnLoad()
	local funcname = "OnLoad"
	self:DebugMsg(funcname, "entry")

    -- Called by in-line code at the end of this file
    -- when the addon is loaded
    -- Variables have not yet been loaded
    assert( not self.VariablesAreLoaded )
    assert( not self.OnLoadCompleted )

	-- load the event types to prepare for RegisterAddonEventType
	self.EventTypes.AS_FILTERS		= self:CopyTable( L.EVENTTYPES.AS_FILTERS )
	self.EventTypes.IN_SPELL_LIST	= self:CopyTable( L.EVENTTYPES.IN_SPELL_LIST )

	self:InitRuntimeData()			-- initialize self.RuntimeData.Stuff\	
	self:RegisterAllEvents()		-- includes registering for ADDON_LOADED event -> OnVariablesLoaded
	self:RegisterChatCommands()		-- setup /ss stuff
	self:Templates_OnInitialize()	-- initialize Templates info for default speeches
	self:ClickHere_Init()			-- setup to support [Click Here] links in chat messages

	-- NOTE: the rest of the initialization does not occur until OnVariablesLoaded below
	self:DebugMsg(funcname, "done")
    self.OnLoadCompleted = true
end


-- after OnInitialize, the main initialization waits until the saved variables areloaded
function SpeakinSpell:OnVariablesLoaded()
	local funcname = "OnVariablesLoaded"
	self:DebugMsg(funcname, "starting")

	-- validate startup logic
	-- this is important to ensure that saved data doesn't get erased by startup initialization
	-- and catch it when it does (after WoW patches change timing of events)
    assert( self.OnLoadCompleted )
    assert( not self.VariablesAreLoaded )
	self.VariablesAreLoaded = true
	
	-- SpeakinSpellSavedData and ForAll will always exist
	-- they'll be empty if no saved data was actually loaded (fresh install or reset)
	
	if	SpeakinSpellSavedData.Version or		-- current test for existance of saved data for versions >= 3.2.2.17
		SpeakinSpellSavedData.EventTable or		-- old data location from before v3.2.2.17
		SpeakinSpellSavedData.SpellTable then	-- old name from v3.1.2.04 and earlier
		-- If the format of the saved data has changed due to upgrading to a newer version
		-- then fix the old saved data for compatibility - also may fix corrupted data, etc.
		self:ValidateAllSavedData()
		self:Templates_OnVariablesLoaded()	-- initialize Templates info for default speeches
	else
		-- new user: load defaults
		self:DebugMsg(funcname, "EventTable does not exist, using defaults")
		self:InitDefaultSavedData() --also calls ValidateAllSavedData
	end	

	-- set up the Options GUI
	-- NOTE: we want to have the Saved Data before we do this
	self:CreateGUI()
	
	-- Print welcome message
	self:ShowWelcomeMessages()
	
	-- trigger a Speech Event for loading
	local DetectedEventStub = {
		-- event descriptors
		name = L["SpeakinSpell Loaded"],
		type = "EVENT",
		-- event-specific data for substitutions
		caster = UnitName("player"),
		target = UnitName("player"),
	}
	self:OnSpeechEvent( DetectedEventStub ) 
	
	-- start network communications, if applicable
	self:Network_Init()
	
	-- start watching for rez events from LibResComm 
	-- this might immediately start wanting to use the speeches
	self:Rez_Init()

	-- Done
	self:DebugMsg(funcname, "done")
end


function SpeakinSpell:OnEnable()
	-- Called when the addon is enabled
	-- self:RegisterAllEvents() -- did this OnInitialize
end


function SpeakinSpell:OnDisable()
    -- Called when the addon is disabled
	-- we don't need to do anything here
end



-- for addons to create new event types
function SpeakinSpell:RegisterAddonEventType( TYPE, SomethingEvents, AsPrefix )
	self.EventTypes.AS_FILTERS		[ TYPE ] = SomethingEvents	-- "Something Events"
	self.EventTypes.IN_SPELL_LIST	[ TYPE ] = AsPrefix			-- "Something Event: "
end


function SpeakinSpell:ToggleSetupGuides()
	self:EnableSetupGuides( not SpeakinSpellSavedData.ShowSetupGuides, false )
end

function SpeakinSpell:EnableSetupGuides( value, IsInit )
	SpeakinSpellSavedData.ShowSetupGuides = value
	if SpeakinSpellSavedData.ShowSetupGuides then
		self:Print( string.format(L["Setup guides are enabled. %s to disable them"], 
			self:MakeClickHereLink(L["[/ss guides]"], "SpeakinSpell:ToggleSetupGuides()") ))
		self:Print( string.format(L["%s to review recent events and speeches"], 
			self:MakeClickHereLink(L["[/ss recent]"], "SpeakinSpell:ReportRecentEvents()") ))
	elseif (not IsInit) then
		self:Print( string.format(L["Setup guides have been disabled. %s to enable them"], 
			self:MakeClickHereLink(L["[/ss guides]"], "SpeakinSpell:ToggleSetupGuides()") ))
	end
end


function SpeakinSpell:ShowWelcomeMessages()
	-- Welcome to SpeakinSpell v3.3.0.07 [Click Here] to edit options
	if SpeakinSpellSavedData.ShowVersionAtLogin then
		self:Print( string.format(L["Welcome to SpeakinSpell v%s %s to edit options"], 
			self.CURRENT_VERSION, self:MakeClickHereLink(L["[Click Here]"], "SpeakinSpell:ShowOptions()")) )
		-- Automatic event detection is On/Off
		if SpeakinSpellSavedData.EnableAllMessages then
			self:Print( L["Automatic event detection and announcements are enabled."] )
		else
			self:Print( L["Automatic event detection and announcements are disabled."] )
			self:Print( L["However, \"/ss macro something\" will still trigger announcements if run manually"] )
		end
	end
	-- Setup guides are enabled [Click Here] to disable them
	SpeakinSpell:EnableSetupGuides( SpeakinSpellSavedData.ShowSetupGuides, true )
end


function SpeakinSpell:ReportRecentEvents()
	self:Print( L["Recent Events Detected..."] )
	for i=SpeakinSpell.MAX.RECENT_HISTORY,1,-1 do
		local de = self.RuntimeData.Recent.Events[i]
		if de then
			self:DoReportDetectedSpeechEvent( de, true, false )
		end
	end
	self:Print( L["Recent Speech Announcements..."] )
	for i=SpeakinSpell.MAX.RECENT_HISTORY,1,-1 do
		local item = self.RuntimeData.Recent.Speeches[i]
		if item then
			self:DoReportDetectedSpeechEvent( item.de, true, false )
			self:Print( item.msg )
		end
	end
end


SpeakinSpell:OnLoad() -- see comments in OnInitialize

